#!/usr/bin/env python
import re
import argparse
import os
import sys
from logging import getLogger, DEBUG, warning
from typing import List, Tuple

help_pattern = re.compile(r'(^[a-zA-Z\-]+):([a-zA-Z\- ]*)##(.*?)$')

logger = getLogger()


def parse_help_from_make(filename) -> List[Tuple[str, str]]:
    """
    Parse help from a makefile

    Args:
        filename: Filename to parse help from

    Returns:
        A list of makefile rules with descriptions, sorted by name
    """
    with open(filename, 'r') as make_in:
        lines = make_in.readlines()
        h_items = []
        for line in lines:
            m = help_pattern.match(line)
            if m:
                h_items.append((m.group(1), m.group(3)))

        h_items = sorted(h_items, key=lambda x: x[0])
        return h_items


def print_help(items: List[Tuple[str, str]]):
    """
    Print out the help for list of items

    Args:
        items: Items to print help for

    Returns:
        None, prints each help item
    """
    print("\n".join([f'{x[0].strip().ljust(20)}:{x[1].strip()}' for x in items]))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--file", default=[], action="append", help="Path to makefile")
    args = parser.parse_args()
    if args.file:
        help_items = []

        for f in args.file:
            if logger.isEnabledFor(DEBUG):
                logger.debug(f)
            if not os.path.exists(f):
                warning(f"Cannot find the file: {f}")
            else:
                help_items.extend(parse_help_from_make(os.path.abspath(f)))
    else:
        help_items = parse_help_from_make(f'{os.path.join(os.getcwd(), "Makefile")}')
    print_help(help_items)


if __name__ == '__main__':
    main()
